/**
***     MathApp.c
***
***     ̃\[XR[h́AG[XgbvTXyhȂǂ̏Ă܂B
***     Copyright (C) 2006 SophiaCradle Incorporated.
***     All rights reserved.
**/

#include "AEEModGen.h"
#include "AEEAppGen.h"
#include "AEEStdLib.h"
#include "AEEShell.h"
#include "common.h"
#include "MathApp.bid"
#include "AEESource.h"
#include <math.h>
#if defined __arm
#include <rt_misc.h>
#include <errno.h>
#include <rt_fp.h>
#endif

#if defined __arm
#pragma import(__use_no_semihosting_swi)
#endif

#define     M_PI            (3.14159265358979323846)

typedef struct MathApp {
    AEEApplet a; // Avbg\̂̐擪o͕K AEEApplet ^ɂ邱ƁB
    Common common; // `p

#if defined __arm
    signed int volatile _errno;
    unsigned int _fpstatus;
#endif
} MathApp;

static boolean MathApp_HandleEvent(MathApp* app, AEEEvent eCode, uint16 wParam, uint32 dwParam);
static boolean OnAppStart(MathApp* app);
static boolean OnAppStop(MathApp* app);
static boolean OnKey(MathApp* app, uint16 key);
static void GetRandAndWrite(MathApp* app);
static void GetRandByISourceAndWrite(MathApp* app);
static void CalculateSinAndWrite(MathApp* app);

#if defined __arm
void __rt_raise(signed int signal, signed int type)
{
    return;
}

signed int volatile* __rt_errno_addr(void)
{
    MathApp* ap = (MathApp*)GETAPPINSTANCE();

    return &ap->_errno;
}

unsigned int* __rt_fp_status_addr(void)
{
    MathApp* ap = (MathApp*)GETAPPINSTANCE();

    return &ap->_fpstatus;
}
#endif

int AEEClsCreateInstance(AEECLSID ClsId, IShell* pIShell, IModule* po, void** ppObj)
{
    *ppObj = NULL;
    if (ClsId == AEECLSID_MATHAPP) {
        if (AEEApplet_New(sizeof(MathApp), ClsId, pIShell, po, (IApplet**)ppObj, (AEEHANDLER)MathApp_HandleEvent, NULL) == TRUE) {
#if defined __arm
            _fp_init();
#endif
            return AEE_SUCCESS;
        }
    }
    return EFAILED;
}

static boolean MathApp_HandleEvent(MathApp* app, AEEEvent eCode, uint16 wParam, uint32 dwParam)
{
    switch (eCode) {
        case EVT_APP_START:
            return OnAppStart(app);

        case EVT_APP_STOP:
            return OnAppStop(app);

        case EVT_KEY:
            return OnKey(app, wParam);
    }
    return FALSE;
}

// AvbgJnƂɌĂяoB
static boolean OnAppStart(MathApp* app)
{
    IShell* shell = app->a.m_pIShell;
    IDisplay* display = app->a.m_pIDisplay;

    COMMON_Init(&app->common, display, shell, 32768);
    COMMON_WriteString(&app->common, "Push 1 Key to test GETRAND\nPush 2 Key to get random bytes by using ISoruce\nPush 3 Key to test type 'double' and sin function\n");
    COMMON_Draw(&app->common);
    return TRUE;
}

// AvbgIƂɌĂяoB
static boolean OnAppStop(MathApp* app)
{
    COMMON_Free(&app->common);
    return TRUE;
}

// L[ꂽƂɌĂяoB
static boolean OnKey(MathApp* app, uint16 key)
{
    switch (key) {
        case AVK_1:
            GetRandAndWrite(app);
            COMMON_Draw(&app->common);
            return TRUE;

        case AVK_2:
            GetRandByISourceAndWrite(app);
            COMMON_Draw(&app->common);
            return TRUE;

        case AVK_3:
            CalculateSinAndWrite(app);
            COMMON_Draw(&app->common);
            return TRUE;

        case AVK_DOWN:
            COMMON_NextLine(&app->common);
            COMMON_Draw(&app->common);
            return TRUE;

        case AVK_UP:
            COMMON_PreviousLine(&app->common);
            COMMON_Draw(&app->common);
            return TRUE;
    }
    return FALSE;
}

// GetRand ֐g̐
static void GetRandAndWrite(MathApp* app)
{
    byte bytes[16];
    char buf[80];
    int i;

    GETRAND(bytes, 16);
    COMMON_WriteString(&app->common, "got random bytes by using GETRAND().\n");
    for (i = 0; i < 16; i++) {
        SPRINTF(buf, "index %2d: %d\n", i, bytes[i]);
        COMMON_WriteString(&app->common, buf);
    }
    return;
}

// ISource C^[tF[Xg̐
static void GetRandByISourceAndWrite(MathApp* app)
{
    byte bytes[16];
    char buf[80];
    ISource* random;
    int i;

    random = NULL;
    ISHELL_CreateInstance(app->a.m_pIShell, AEECLSID_RANDOM, (void**)&random);
    ISOURCE_Read(random, (char*)bytes, 16);
    ISOURCE_Release(random);

    COMMON_WriteString(&app->common, "got random bytes by using ISource.\n");
    for (i = 0; i < 16; i++) {
        SPRINTF(buf, "index %2d: %d\n", i, bytes[i]);
        COMMON_WriteString(&app->common, buf);
    }
    return;
}

// _Ɛw֐gB
static void CalculateSinAndWrite(MathApp* app)
{
    int i;
    int j;
    AECHAR wstring[32];
    char string[32];

    COMMON_WriteString(&app->common, "calculate sin function.\n");
    for (i = 0; i <= 90; i += 15) {
        SPRINTF(string, "sin(%2d)=", i);
        COMMON_WriteString(&app->common, string);
        FLOATTOWSTR(sin(i * M_PI / 180), wstring, 32);
        WSTRTOSTR(wstring, string, 32);
        for (j = 0; j < 32; j++) {
            if (string[j] != ' ') {
                break;
            }
        }
        COMMON_WriteString(&app->common, string + j);
        COMMON_WriteString(&app->common, "\n");
    }
    return;
}
